[include version_include.man]
[manpage_begin {keyl} n [vset version]]
[moddesc {NaviServer Built-in Commands}]

[titledesc {Keyed Lists}]

[description]
Keyed lists are an extension of Tcl provided by the TclX package.  They are an alternative to using 
ns_set, but we highly recommend using ns_set instead of keyed lists where possible.
The commands provided by NaviServer were taken directly from a 1995 version of the TclX package.

[section {COMMANDS}]

[list_begin definitions]

[call [cmd keylget] [arg listvar] [opt [arg key]] [opt [arg retvar]]]

This command returns the value associated with key from the keyed list in the variable listvar.   
If retvar is not specified, then the value will be returned as the result of the command.  In this
case, if key is not found in the list, an error will result.

[para]
If retvar is specified and key is in the list, then the value is returned in the variable retvar
and the command returns 1.  Otherwise, the command will return 0, and retvar will be left unchanged.

[para]
If retvar is an empty string, the value is not returned, allowing the Tcl programmer to determine
if a key is present in a keyed list without setting a variable as a side-effect.
If key is omitted, then a list of all the keys in the keyed list is returned similar to


[call [cmd keyldel] [arg listvar] [arg key]]

Delete key and value from keyed list

[para]
This command deletes the key specified by key from the keyed list in listvar.  It removes 
both the key and the value from the keyed list.

[para]
Keyed lists are an extension of Tcl provided by the TclX package.  They are an alternative to 
using ns_set, but we highly recommend using ns_set instead of keyed lists.  The commands provided
by NaviServer were taken directly from a 1995 version of the TclX package.

[call [cmd keylset] [arg listvar] [arg key] [arg value] [opt [arg {key value...}]]]
Set a key-value pair in a keyed list

[para]
This command sets one or more key-value pairs in the keyed list listvar.

[call [cmd keylkeys] [arg listvar] [opt [arg key]]]

Return the keys for a keyed list.

[para]
This command returns a list of keys for the keyed list listvar.  If key is specified, 
the values for that key are returned instead.

[list_end]

[section EXAMPLES]

[example_begin]
 % keylset mylist a b c d e f
[example_end]

[example_begin]
 % keylkeys mylist
 a c e
[example_end]

[example_begin]
 % keyldel mylist c
[example_end]

[example_begin]
 % keylkeys mylist
 a e
[example_end]

[example_begin]
 % keylset mylist a 1 {b c} 2
[example_end]

[example_begin]
 % keylkeys mylist
 a {b c}
[example_end]

[example_begin]
 % keylkeys mylist a
 1
[example_end]


[see_also nsd ns_set]
[keywords "global built-in" TclX keyed-list ]

[manpage_end]

