[include version_include.man]
[manpage_begin ns_adp_parse n [vset version]]
[moddesc {NaviServer Built-in Commands}]

[titledesc {Parse an ADP file or string and return its output}]

[description]

This function processes the specified ADP file or string and returns
the result as a string. Processing a second ADP from inside an ADP
is normally best done with ns_adp_include as that command
resolves relative pathnames passed to it.  Also note that
ns_adp_parse will ignore any directives to turn on streaming.
Tcl_Eval is used to evaluate the Tcl commands in the ADP.

[para]
Although the <% ... %> syntax is allowed in the string,
if you have this embedded in an ADP, you have to be careful in
constructing the string that you do not prematurely terminate an
enclosing script.

[section {COMMANDS}]

[list_begin definitions]

[call [cmd ns_adp_parse] \
	[opt [option {-cwd path}]] \
	[opt [option {-file}]] \
	[opt [option {-safe}]] \
	[opt [option {-string}]] \
	[opt [option {-tcl}]] \
	[opt {args ...}]]

[list_end]

[para]
The option [opt  [option -cwd]] can be used to set the current working directory
for this ADP parse operation

[para]
The option  [opt [option -file]] can be used to parse ADP contained in
the given file.  This use is similar to that of
[cmd ns_adp_include] except the result is returned by the command instead of automatically
being appended to the output stream.

[para]
The option [opt [option -safe]] can be used to set the ADP safe flag.

[para]
The option [opt [option -string]] is provided for backwards compatibility, but
does nothing since string mode is the default mode.

[para]
The option [opt [option -tcl]] can be provided to process a Tcl file.

[para] Additional arguments can be provided, which are passed to the
ADP evaluation.  The arguments can be accessed within the ADP using
the [cmd ns_adp_argc], [cmd ns_adp_argv], and [cmd ns_adp_bind_args]
commands.

[para]
If this call is a nested ADP evaluation (where one ADP calls another), an error will
be thrown if the depth exceeds 256, the maximum number of nested ADP evaluations.

[section EXAMPLES]

[example_begin]
 % ns_adp_parse {2 + 2 = <%= [lb]expr 2 + 2[rb] %> !!!}
 2 + 2 = 4 !!!
[example_end]

[example_begin]
 % ns_adp_parse -file foo.tcl
[example_end]

[see_also ns_adp ns_adp_argc ns_adp_argv ns_adp_include ns_adp_bind_args]
[keywords "server built-in" ADP]

[manpage_end]

