[include version_include.man]
[manpage_begin ns_getform n [vset version]]
[moddesc {NaviServer Built-in Commands}]

[titledesc {Return an ns_set that contains all of the query data that was part of the HTTP request}]

[description]

This command returns a handle to an [term ns_set] which contains all
of the query data from the current HTTP request.  The resulting data
includes query parameters passed in the URL as well as multipart form
data.  If there is no form data, an empty [term ns_set] is created.

[section {COMMANDS}]

[list_begin definitions]

[call [cmd ns_getform ] \
    [opt [option "-fallbackcharset [arg charset]"]] \
    [opt [arg defaultcharset]] \
]

[para]
The command [cmd ns_getform] gets form data from GET or POST requests
and processes the data either in the spool file or in memory.

[para]
For GET requests [cmd "ns_conn form"] and [cmd  ns_getform] are
identical. The command handles content-types
[term application/x-www-form-urlencoded] and
[term multipart/form-data].

[para]
For uploaded files two additional elements are created in the ns_set,
named input_name[term .content-type] and
input_name[term .tmpfile]. The filename for the temporary file is
generated by [cmd ns_mktemp], the file is deleted automatically,
when the connection is closed.

[para]

In case in the provided form data contains invalid UTF-8 text, an
exception is raised (with exception code NS_INVALID_UTF8). When the
global or per-server configuration variable [term formfallbackcharset]
is provided, a second decoding attempt is made with this charset and a
warning is generated when invalid UTF-8 data is encountered.  Note
that the second attempt using the fallback charset is made as well,
when an invalid charset was provided with in the content-type header
field.

By using the option [option "-fallbackcharset [arg charset]"]
the fallback charset can be specified explicitly, which overrides
the configuration parameters.

[example_begin]
  ns_getform -fallbackcharset windows-1252
[example_end]

[para]

When the provided content-type header field type contains no charsetl
specification, the configuration variable [term urlcharset] can be
used.  In order to override the configuration value, the parameter
[arg defaultcharset] can be passed to [cmd ns_getform].  The parameter
should not be confused with the fallback charset which is only used
when decoding results in invalid UTF-8.

[example_begin]
  ns_getform iso8859-1
[example_end]

[list_end]

[keywords form urlcharset formfallbackcharset content-type]
[see_also ns_getcontent ns_getformfile ns_queryget ns_set ns_mktemp \
        ns_conn ns_atclose ns_parsequery]


[manpage_end]
