[include version_include.man]
[manpage_begin ns_urlencode n [vset version]]
[moddesc {NaviServer Built-in Commands}]



[titledesc {Encode a string to its URL-encoded representation}]

[description]

 This command encodes a string according to the rules for URL encoding
 defined in RFC 3986 Uniform Resource Identifier (URI), (or RFC 1738,
 when NaviServer was configured to use this older standard).

 ns_urlencode codes essentially all non-alphanumeric characters in the
 specified components.  The epcial octets are encoded by a "%"
 followed by the two-character hexa- decimal representation for the
 octet.

[section {COMMANDS}]

[list_begin definitions]

 [call [cmd ns_urlencode] \
     [opt [option "-charset [arg charset]"]] \
     [opt [option "-part [arg part]"]] \
     [opt [arg "--"]] \
     [arg components]]

[list_end]

The option [option -charset] can be used to specify the character set of
the encode operation. Option [arg part] is used to specify the
[arg query] (default) or [arg path] encoding. In the query mode, the
components will be concatenated via "&", in the path mode the
components are concatenated with "/".

[para] The part argument can be as well used for more percent
encodings such as the cookie encoding (value "cookie", RFC 6265) or
the oauth1 encoding (value "oauth1") as defined in RFC 5849 Section
3.6 for the construction of the * signature base string and the
"Authorization" header field.

[section {EXAMPLES}]

The following example encodes the Tcl dict as
application/x-www-form-urlencoded POST data
(https://www.w3.org/TR/html401/interact/forms.html#h-17.13.4)

[example_begin]
 set data {
   first_name  John
   second_name Doe
   data        "Hello World"
 }
 
 set post_data {}
 foreach {key value} $data {
     lappend post_data "[lb]ns_urlencode -part query $key[rb]=[lb]ns_urlencode $value[rb]"
 }
 set post_data [lb]join $post_data &[rb]
[example_end]

The second example shows encodings of path segments based on example 1
and 2 of https://www.w3.org/Addressing/URL/4_URI_Recommentations.html

[example_begin]
 % set e1 {albert bertram marie-claude}
 % ns_urlencode -part path {*}$e1
 albert/bertram/marie-claude
 
 % set e2 {albert bertram/marie-claude}
 % ns_urlencode -part path {*}$e2
 albert/bertram%2fmarie-claude
[example_end]

[see_also nsd ns_urldecode ns_charsets ns_urlcharset ns_http]
[keywords "global built-in" encoding charset url]

[manpage_end]

