[include version_include.man]
[manpage_begin {nscp} n [vset version]]
[moddesc   {NaviServer Modules}]

[titledesc {NaviServer Control Port Guide}]

[description]


[section {Introduction}]

NaviServer includes a control port interface that can be enabled with
the nscp module. This control port interface allows you to telnet to a
specified host and port where you can administer the server and
execute database commands while the server is running.


[para]
The control port interface is a Tcl interface. Within it, you can
execute any Tcl command that Tcl libraries and ADP can; the only
exceptions being the ns_conn commands.


[para]
To administer a server with the control port interface while it is
running, the nscp module must be loaded into the server. To set
permissions for the server, the nsperm module must also be loaded. To
execute database commands while the server is running, you must also
configure database services for the server, including database drivers
and pools.


[section {Configuring the Control Port Interface}]


[para]
An example configuration for the control port interface is shown
below. Three sections of the configuration file are included. The nscp
module is loaded into the /modules section for $server. The
/module/nscp section defines the control port parameters, and the
/module/nscp/users section defines the users who can log into the
control port.


[example_begin]
 #
 # Control port -- nscp
 #
 ns_section ns/server/${server}/module/nscp {
   ns_param   port            9999        ;# Control port listens on port 9999
   ns_param   address         "127.0.0.1" ;# For security, use 127.0.0.1 only
 }
 
 # Control port users
 ns_section ns/server/${server}/module/nscp/users {
   #
   # The default password for nsadmin is "x". You should change it.
   # type "ns_crypt newpassword salt" and put the encrypted string
   # below.
   
   ns_param user "nsadmin:t2GqvvaiIUbF2:" ;# sample user="nsadmin", pw="x".
 }
 
 ns_section ns/server/${server}/modules {
   # ...
   ns_param nscp nscp.so
 }
[example_end]



[para]
Some things to notice about the configuration are:

[list_begin itemized]

[item]  The port parameter defines the port that the interface will listen
     on. The default is 9999.


[item]  The address parameter defines the hostname to bind to. The default
     is 127.0.0.1 so that you can only connect to the control port if
     you are already logged on to the computer where the server is
     running.

[item]  The user parameter defines users who can log into the control
     port. There are three semicolon-separated fields. The first is
     username (nsadmin); the second is a Unix-like password hash.  You
     can create a password by typing "ns_crypt mypasswd salt".  As with
     Unix, only the first four characters are used.

[list_end]



[para]
See the Configuration Reference for complete information on
configuration.


[section {Using the Control Port Interface}]

[para]
To use the control port interface, follow these steps:

[list_begin enum]

[enum] Telnet to the address and port number defined in the
     /module/nscp section for the server. For example:

[example_begin]
 telnet localhost 9999
[example_end]


[para]


     The session responds with:


[example_begin]
 Trying 127.0.0.1...
 Connected to localhost.
 Escape character is '^]'.
 login:
[example_end]


[enum] At the login prompt, type a valid username. The username must be
     one of the usernames defined in the /module/nscp/users section of
     the configuration file. For example:


[example_begin]
  login: nsadmin
[example_end]

The session responds with:


[example_begin]
 password:
[example_end]


[enum] At the password prompt, type the password associated with the
     username you specified.  The default is "x".


[para]
The session responds with:

[example_begin]
 Welcome to server1 running at /usr/local/ns/bin/nsd (pid 22413)
 NaviServer/4.99.29 for linux built on MON/DAY/YEAR at HH:MM:SS


 server1:nscp 1>
[example_end]



[enum] At the prompt you can type nearly any Tcl command
     available to NaviServer Tcl libraries and ADPs.

[enum] When you're done, just type "exit" and you will be logged off.


[list_end]


[section {Useful Commands}]

Type "lsort [lb]info commands[rb]" for a complete list of commands available
to you.  You can type nearly any Tcl command available to NaviServer
Tcl libraries and ADPs, including the complete Tcl core and
nearly any ns_* commands.  Type "lsort [lb]info commands ns*[rb]" for a
sorted list of available NaviServer Tcl commands.


[para]
Useful commands:


[list_begin itemized]

[item] ns_shutdown  -- Shuts down the server.

[item] ns_info uptime  -- How long the server has been running.

[item] ns_info threads  -- How many threads are running

[item] exit -- Exit the control port.


[list_end]

[see_also admin-maintenance]
[keywords module nscp configuration]

[manpage_end]
